package com.asreader.asfinger;

import android.annotation.SuppressLint;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

import com.asreader.asfingersdk.AsFingerSDK;
import com.asreader.asfingersdk.event.AsFingerEventListener;
import com.asreader.asfingersdk.type.AsFingerConnectionType;
import com.asreader.asfingersdk.type.DeviceType;

import java.nio.charset.Charset;
import java.util.Timer;
import java.util.TimerTask;

import static com.asreader.asfingersdk.AsFingerSDK.FIRM_VERSION;

public class BluetoothChat extends AppCompatActivity implements View.OnClickListener, RadioGroup.OnCheckedChangeListener {

    //Battery Level
    private static final int EMPTY_BATTERY = 0;
    private static final int LOW_BATTERY = 1;
    private static final int SLIGHTLY_HIGH_BATTERY = 2;
    private static final int HIGH_BATTERY = 3;
    private static final int FULL_BATTERY = 4;
    private static final String TAG = "BluetoothChat";

    // Encoding
    private Charset mCharset_ASCii = Charset.forName("ASCII");
    private Charset mCharset_UTF8 = Charset.forName("UTF-8");
    private Charset mCharset_ShiftJIS = Charset.forName("Shift_JIS");

    private AsFingerSDK mAsFingerSDK;
    // Layout Views
    private ListView mConversationView;
    // Array adapter for the conversation thread
    private ArrayAdapter<String> mConversationArrayAdapter;
    private ImageView mIvBattery;
    private Timer timerCheckBattery;
    private ImageView mAppLogo;
    private ImageButton mIbSetting;
    private Button mBtStartScan;
    private Button mBtStopScan;
    private Button mBtClear;
    private TextView mTvDemoVersion;
    private RadioButton mRbAscii;
    private RadioGroup mRgCharacterset;
    private Charset mCharset = mCharset_ASCii;

    private int mBattery;
    private boolean isConnect = false;
    private boolean isGetFirmwareVersion = false;
    private BluetoothAdapter mBtAdapter;
    private static final int REQUEST_ENABLE_BT = 2;
//    public static BluetoothDevice mLastConnectedBluetoothDevice;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        mBtAdapter = BluetoothAdapter.getDefaultAdapter();

        initView();

        if (mBtAdapter == null) {
            return;
        }
        if (!mBtAdapter.isEnabled()) mBtAdapter.enable();

        mAsFingerSDK = AsFingerSDK.getInstance();
        mAsFingerSDK.setActivity(this);
        mAsFingerSDK.setAsFingerEventListener(mAsFingerEventListener);

        connectLastDevice();

//        // If the adapter is null, then Bluetooth is not supported
//        if (mBtAdapter == null) {
//            Toast.makeText(this, getResources().getString(R.string.toast_bluetooth_is_off),
//                    Toast.LENGTH_LONG).show();
//            finish();
//            return;
//        }
    }

    @Override
    protected void onStart() {
        super.onStart();
//        if (!mBtAdapter.isEnabled()) {
//
//            Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
//            startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
//        }else {
//            setupChat();
//        }
        if (mBtAdapter == null) {
            return;
        }
        if (!mBtAdapter.isEnabled()) mBtAdapter.enable();

        setupChat();
    }

    private void setupChat() {
        mAsFingerSDK.setActivity(this);
        mAsFingerSDK.setAsFingerEventListener(mAsFingerEventListener);
        if (mAsFingerSDK.connectedDevice() != null) {
            isConnect = true;
            startCheckBattery();
        } else {
            isConnect = false;
            mAppLogo.setImageResource(R.drawable.asreader_log_off);
            stopCheckBattery();
            mIvBattery.setImageResource(R.drawable.battery);
        }
    }

    private void connectLastDevice() {
        SharedPreferences ps = getSharedPreferences("BluetoothDevice", Context.MODE_PRIVATE);
        String deviceAddress = ps.getString("address", "");
        int type = ps.getInt("type", 0);
        if (!deviceAddress.equals("") && type != 0) {
            BluetoothDevice mLastConnectedBluetoothDevice = mBtAdapter.getRemoteDevice(deviceAddress);
            mAsFingerSDK.connect(mLastConnectedBluetoothDevice, DeviceType.setType(type));
        }
    }


    private void initView() {
        mIbSetting = (ImageButton) findViewById(R.id.ib_setting);
        mIvBattery = (ImageView) findViewById(R.id.iv_battery);
        mAppLogo = (ImageView) findViewById(R.id.iv_applogo);
        mBtStartScan = (Button) findViewById(R.id.bt_startscan);
        mBtStopScan = (Button) findViewById(R.id.bt_stopscan);
        mBtClear = (Button) findViewById(R.id.bt_clear);
        mTvDemoVersion = (TextView) findViewById(R.id.tv_demo_version);
        mRgCharacterset = (RadioGroup) findViewById(R.id.rg_characterset);
        mRbAscii = (RadioButton) findViewById(R.id.rbascii);
        mRbAscii.setChecked(true);
        // Initialize the array adapter for the conversation thread
        mConversationArrayAdapter = new ArrayAdapter<String>(this,
                R.layout.message);
        mConversationView = (ListView) findViewById(R.id.in);
        mConversationView.setAdapter(mConversationArrayAdapter);
        mBtClear.setOnClickListener(this);
        mIbSetting.setOnClickListener(this);
        mBtStopScan.setOnClickListener(this);
        mBtStartScan.setOnClickListener(this);
        mRgCharacterset.setOnCheckedChangeListener(this);
        displayVersion();


    }

    private void displayVersion() {
        String versionName = "";
        String packageName = getPackageName();
        try {
            versionName = getPackageManager().getPackageInfo(packageName, PackageManager.GET_META_DATA).versionName;
        } catch (PackageManager.NameNotFoundException e) {
            versionName = "";
        }
        mTvDemoVersion.setText(getResources().getString(R.string.app_version) + " " + versionName);
    }

    @SuppressLint("WrongConstant")
    private void checkBluetoothConnect() {
        if (!isConnect) {
            Toast.makeText(BluetoothChat.this, getString(R.string.not_connected), 0).show();
            return;
        }
    }

    @Override
    protected void onResume() {
        if (mAsFingerSDK != null && isConnect) {
            startCheckBattery();
        }
        super.onResume();
    }

    @Override
    protected void onPause() {
        if (mAsFingerSDK != null) {
            stopCheckBattery();
        }
        super.onPause();

    }

    @Override
    protected void onStop() {
        super.onStop();

    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        // Stop the Bluetooth chat services
        if (mAsFingerSDK != null) {
            isConnect = false;
            mAsFingerSDK.disconnect();
            mAsFingerEventListener = null;
        }

        System.exit(0);
    }

    private void startCheckBattery() {

        if (timerCheckBattery != null)
            return;

        int interval = 10 * 1000;
        timerCheckBattery = new Timer();
        timerCheckBattery.schedule(new TimerTask() {

            @Override
            public void run() {
                mAsFingerSDK.getBattery();
            }

        }, 1000, interval);
    }

    private void stopCheckBattery() {
        if (timerCheckBattery == null)
            return;
        timerCheckBattery.cancel();
        timerCheckBattery = null;
    }

    private int checkBatteryStatus(int battery) {
        if (battery != 0) {
            if (battery > 75) {
                return FULL_BATTERY;
            } else if (battery > 50 && battery <= 75) {
                return HIGH_BATTERY;
            } else if (battery > 25 && battery <= 50) {
                return SLIGHTLY_HIGH_BATTERY;
            } else if (battery > 5 && battery <= 25) {
                return LOW_BATTERY;
            } else if (battery <= 5) {
                return EMPTY_BATTERY;
            }
        } else {
            Toast.makeText(this, getResources().getString(R.string.no_power),
                    Toast.LENGTH_LONG).show();
            return -1;
        }
        return -1;


    }

    private void checkBattery() {

        switch (mBattery) {
            case EMPTY_BATTERY:
                mIvBattery.setImageResource(R.drawable.battery0);
                break;
            case LOW_BATTERY:
                mIvBattery.setImageResource(R.drawable.battery1);
                break;
            case SLIGHTLY_HIGH_BATTERY:
                mIvBattery.setImageResource(R.drawable.battery2);
                break;
            case HIGH_BATTERY:
                mIvBattery.setImageResource(R.drawable.battery3);
                break;
            case FULL_BATTERY:
                mIvBattery.setImageResource(R.drawable.battery4);
                break;
            default:
                return;
        }

    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.ib_setting:
                Intent intent = new Intent(this, Setting.class);
                startActivity(intent);
                break;
            case R.id.bt_clear:
                mConversationArrayAdapter.clear();
                break;
            case R.id.bt_startscan:
                if (isConnect) {
                    mAsFingerSDK.stopScan();
                    try {
                        Thread.sleep(200);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    mAsFingerSDK.startScan();
                } else {
                    checkBluetoothConnect();
                }

                break;
            case R.id.bt_stopscan:
                if (isConnect) {
                    mAsFingerSDK.stopScan();
                } else {
                    checkBluetoothConnect();
                }
                break;

        }
    }

    private void encodingType(Charset charset) {
        mCharset = charset;
    }

    @Override
    public void onCheckedChanged(RadioGroup group, int checkedId) {
        switch (checkedId) {
            case R.id.rbascii:
                encodingType(mCharset_ASCii);
                break;
            case R.id.rbshiftjis:
                encodingType(mCharset_ShiftJIS);
                break;
            case R.id.rbutf8:
                encodingType(mCharset_UTF8);
                break;
        }
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            moveTaskToBack(false);
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == AppCompatActivity.RESULT_OK) {
            setupChat();
        }
    }

    private AsFingerEventListener mAsFingerEventListener = new AsFingerEventListener() {

        @Override
        public void onStateChanged(AsFingerConnectionType connectionType) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    switch (connectionType) {
                        case connected:
                            isConnect = true;
                            startCheckBattery();
                            Toast.makeText(BluetoothChat.this,
                                    getResources().getString(R.string.toast_connect_device) + " " + mAsFingerSDK.connectedDevice().getName(), Toast.LENGTH_LONG).show();
                            break;
                        case disconnected:
                            isConnect = false;
                            mAppLogo.setImageResource(R.drawable.asreader_log_off);
                            stopCheckBattery();
                            mIvBattery.setImageResource(R.drawable.battery);
                            Toast.makeText(BluetoothChat.this,
                                    getResources().getString(R.string.toast_disconnect_device), Toast.LENGTH_LONG).show();
                            break;
                        case connecting:
                            Toast.makeText(BluetoothChat.this,
                                    getResources().getString(R.string.toast_connecting_device), Toast.LENGTH_LONG).show();
                            break;
                    }
                }
            });

        }

        @Override
        public void receivedBarcodeData(byte[] barcodeData) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    String str = "";
                    try {
                        str = new String(barcodeData, mCharset);
                    } catch (Exception e) {
                    }
                    String finalStr = str;
                    mConversationArrayAdapter.add(finalStr);
                }
            });


        }

        @Override
        public void receivedBattery(int battery) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    mBattery = checkBatteryStatus(battery);
                    checkBattery();
                    mAppLogo.setImageResource(R.drawable.asreader_log_on);

                }
            });
        }

        @Override
        public void receivedFirmwareVersion(String firmwareVersion) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    if (isGetFirmwareVersion) {
                        isGetFirmwareVersion = false;
                        return;
                    } else {
                        mConversationArrayAdapter.add(FIRM_VERSION + firmwareVersion);
                    }

                }
            });
        }

        @Override
        public void receivedData(byte[] data) {

        }

        @Override
        public void receivedDevice(BluetoothDevice device) {

        }

        @Override
        public void receivedFoundDeviceFinished() {

        }


    };


}
