package com.asreader.asfinger;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.os.Bundle;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;

import com.asreader.asfingersdk.AsFingerSDK;
import com.asreader.asfingersdk.event.AsFingerEventListener;
import com.asreader.asfingersdk.type.AsFingerConnectionType;

import static com.asreader.asfingersdk.AsFingerSDK.FIRM_VERSION;

public class InformationActivity extends AppCompatActivity implements View.OnClickListener {

    private static final String TAG = "InformationActivity";
    private ListView mLvInformation;
    private ArrayAdapter<String> mInformationAdapter;
    private ImageButton mIbBack;
    private AsFingerSDK mAsFingerSDK;
    private BluetoothAdapter mBtAdapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_information);
        mBtAdapter = BluetoothAdapter.getDefaultAdapter();

        initView();

        mAsFingerSDK = AsFingerSDK.getInstance();

    }

    private void initView() {
        mIbBack = (ImageButton) findViewById(R.id.ib_back);
        mLvInformation = (ListView) findViewById(R.id.lv_information);
        mInformationAdapter = new ArrayAdapter<String>(this,
                R.layout.message);
        mLvInformation.setAdapter(mInformationAdapter);
        mIbBack.setOnClickListener(this);


    }


    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.ib_back:
                finish();
                break;
        }
    }

    @Override
    protected void onStart() {
        super.onStart();
        if (mBtAdapter == null) {
            return;
        }
        if (!mBtAdapter.isEnabled()) mBtAdapter.enable();

        mAsFingerSDK.setActivity(this);
        mAsFingerSDK.setAsFingerEventListener(mAsFingerEventListener);
        if (mAsFingerSDK.connectedDevice() == null) {
            return;
        } else {
            mAsFingerSDK.getFirmwareVersion();
        }
    }


    @Override
    protected void onResume() {
        super.onResume();
        if (mAsFingerSDK.connectedDevice() == null) {
            return;
        } else {
            try {
                Thread.sleep(200);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            if (mInformationAdapter.getPosition(mAsFingerSDK.getSdkVersion()) == -1)
                mInformationAdapter.add(mAsFingerSDK.getSdkVersion());

        }

    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mAsFingerEventListener = null;
    }


    private AsFingerEventListener mAsFingerEventListener = new AsFingerEventListener() {

        @Override
        public void onStateChanged(AsFingerConnectionType connectionType) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    switch (connectionType) {
                        case connected:
                            Toast.makeText(InformationActivity.this,
                                    getResources().getString(R.string.toast_connect_device) + " " + mAsFingerSDK.connectedDevice().getName(), Toast.LENGTH_LONG).show();
                            break;
                        case disconnected:
                            Toast.makeText(InformationActivity.this,
                                    getResources().getString(R.string.toast_disconnect_device), Toast.LENGTH_LONG).show();
                            break;
                        case connecting:
                            Toast.makeText(InformationActivity.this,
                                    getResources().getString(R.string.toast_connecting_device), Toast.LENGTH_LONG).show();
                            break;
                    }
                }
            });

        }

        @Override
        public void receivedBarcodeData(byte[] barcodeData) {
        }

        @Override
        public void receivedBattery(int battery) {

        }

        @Override
        public void receivedFirmwareVersion(String firmwareVersion) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    if (mInformationAdapter.getPosition(FIRM_VERSION + " " + firmwareVersion) == -1)
                        mInformationAdapter.add(FIRM_VERSION + " " + firmwareVersion);
                }
            });

        }

        @Override
        public void receivedData(byte[] data) {

        }

        @Override
        public void receivedDevice(BluetoothDevice device) {

        }

        @Override
        public void receivedFoundDeviceFinished() {

        }


    };

}
